**Free
Ctl-Opt NoMain;
//
// LUCase
// ------
// Functions to convert a string to Lower or Upper case.
//
// Jesper Wachs, Version 3, July 2018.
//   Converted to Totally Free RPG.   
//
// Jesper Wachs, Version 2, May 2018.
//   Changed to retrieve upper- and lowercase tables and use them
//   with XLate for improving speed when converting strings to
//   upper and lower case.
//
// Jesper Wachs, Version 1, August 2016.
//   Build to call 'QlgConvertCase' to support all chars.
//
// The source member are delivered 'as is'.
//
// Neither Jesper Wachs nor anyone else who has been involved in
// the creation, production or delivery of this product shall be
// liable for any direct, indirect, consequential or incidental
// damages (including damages for loss of business profits,
// business interruption, loss of business information, and the
// like) arising out of the use or inability to use such product
// even if Jesper Wachs has been advised of the possibility of
// such damages.
//
// Copymember for exported functions.
// ----------------------------------
/copy qrpglejw,lucase_pr

//
// Prototypes for local functions.
// -------------------------------
Dcl-PR Convert VarChar(1024);
  iString VarChar(1024) Const;
  iUpLow Packed(1:0) Const;
End-PR;

Dcl-PR GetTables;
End-PR;

//
// Work fields.
// ------------
Dcl-S lBaseT Char(256) Inz(x'-
                            000102030405060708090a0b0c0d0e0f-
                            101112131415161718191a1b1c1d1e1f-
                            202122232425262728292a2b2c2d2e2f-
                            303132333435363738393a3b3c3d3e3f-
                            404142434445464748494a4b4c4d4e4f-
                            505152535455565758595a5b5c5d5e5f-
                            606162636465666768696a6b6c6d6e6f-
                            707172737475767778797a7b7c7d7e7f-
                            808182838485868788898a8b8c8d8e8f-
                            909192939495969798999a9b9c9d9e9f-
                            a0a1a2a3a4a5a6a7a8a9aaabacadaeaf-
                            b0b1b2b3b4b5b6b7b8b9babbbcbdbebf-
                            c0c1c2c3c4c5c6c7c8c9cacbcccdcecf-
                            d0d1d2d3d4d5d6d7d8d9dadbdcdddedf-
                            e0e1e2e3e4e5e6e7e8e9eaebecedeeef-
                            f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff-
                            ');
Dcl-S lFirstTime Ind Inz(*Off);
Dcl-S lLoT Char(256);
Dcl-S lUpT Char(256);

//
// Exported funtions.
// ------------------
//
// LCase
// -----
// Convert a string to lowercase taking national characters into
// account.
//
Dcl-Proc LCase Export;
Dcl-PI LCase VarChar(1024);
  IString VarChar(1024) Const;
End-PI;

Dcl-S oString VarChar(1024);

  If lFirstTime = *Off;
    CallP GetTables();
  EndIf;

  %Len(oString) = %Len(iString);
  oString = %xlate(lBaseT:lLoT:iString);

  Return oString;

End-Proc LCase;
//
// ResetCase
// ---------
// Reset the flag for tables read. To be used if the job CCSID is
// changed for the tables to be recontructed.
//
Dcl-Proc ResetCase Export;
Dcl-PI ResetCase;
End-PI;

  lFirstTime = *Off;

  Return;

End-Proc ResetCase;
//
// UCase
// -----
// Convert a string to uppercase taking national characters into
// account.
//
Dcl-Proc UCase Export;
Dcl-PI UCase VarChar(1024);
  IString VarChar(1024) Const;
End-PI;

Dcl-S oString VarChar(1024);

  If lFirstTime = *Off;
    CallP GetTables();
  EndIf;

  %Len(oString) = %Len(iString);
  oString = %xlate(lBaseT:lUpT:iString);

  Return oString;

End-Proc UCase;
//
// Internal functions.
// -------------------
//
// Convert
// -------
// Internal function that handles call to API for conversion.
//
Dcl-Proc Convert;
Dcl-PI Convert VarChar(1024);
  iString VarChar(1024) Const;
  iUpLow Packed(1:0) Const;
End-PI;

Dcl-PR QlgCvtCase ExtProc('QlgConvertCase');
  CtrlBlock Const LikeDS(FRCB);
  inString Char(32767) Const Options(*VARSIZE);
  OutString Char(32767) Options(*VARSIZE);
  nLength Int(10:0) Const;
  apiError Char(16);
End-PR;

Dcl-S oString Char(32767);
Dcl-S APIErrorDS Char(16) Inz(*AllX'00');

Dcl-DS FRCB Qualified;
  ReqType Int(10:0);
  CCSID Int(10:0);
  CvtTo Int(10:0);
  Reserved Char(10);
End-DS;

Dcl-DS CtrlBlock LikeDS(FRCB);

  CtrlBlock = *Allx'00';
  APIErrorDS = *AllX'00';
  CtrlBlock.ReqType = 1;
  CtrlBlock.CvtTo = iUpLow;

  CallP QlgCvtCase(CtrlBlock:
                   iString:
                   oString:
                   %Len(iString):
                   APIErrorDS);

  Return %SubSt(oString: 1: %Len(iString));

End-Proc Convert;
//
// GetTables
// ---------
// Gets the uppercase and lowercase tables to use with the
// opcode XLate to convert strings to the desired case.
//
Dcl-Proc GetTables;
Dcl-PI GetTables;
End-PI;

  lLoT = Convert(lBaseT: 1);
  lUpT = Convert(lBaseT: 0);

  lFirstTime = *On;

End-Proc GetTables;
